import { html, css, LitElement } from 'lit';
import { property, state, query } from 'lit/decorators.js';

var __defProp$2 = Object.defineProperty;
var __getOwnPropDesc$1 = Object.getOwnPropertyDescriptor;
var __decorateClass$1 = (decorators, target, key, kind) => {
  var result = kind > 1 ? void 0 : kind ? __getOwnPropDesc$1(target, key) : target;
  for (var i = decorators.length - 1, decorator; i >= 0; i--)
    if (decorator = decorators[i])
      result = (kind ? decorator(target, key, result) : decorator(result)) || result;
  if (kind && result)
    __defProp$2(target, key, result);
  return result;
};
const LabelMixin = (labelSlotName, superClass) => {
  class LabelMixinClass extends superClass {
    constructor() {
      super(...arguments);
      this.labelSlotHasContent = false;
    }
    connectedCallback() {
      super.connectedCallback();
      if (!this.label) {
        console.warn(this.tagName + " needs a `label`");
      }
    }
    labelSlotChanged(e) {
      this.labelSlotHasContent = e.target.assignedNodes({ flatten: true }).length > 0;
    }
    renderLabel() {
      return html`
        ${this.labelSlotHasContent === false ? html`<span class="label">${this.label}</span>` : ""}
        <slot
          class="label"
          style=${this.labelSlotHasContent ? "" : "visibility: hidden"}
          name=${labelSlotName ? labelSlotName : ""}
          @slotchange=${this.labelSlotChanged}></slot>
      `;
    }
  }
  __decorateClass$1([
    property({ type: String })
  ], LabelMixinClass.prototype, "label", 2);
  __decorateClass$1([
    state()
  ], LabelMixinClass.prototype, "labelSlotHasContent", 2);
  return LabelMixinClass;
};

var __defProp$1 = Object.defineProperty;
var __getOwnPropSymbols = Object.getOwnPropertySymbols;
var __hasOwnProp = Object.prototype.hasOwnProperty;
var __propIsEnum = Object.prototype.propertyIsEnumerable;
var __defNormalProp = (obj, key, value) => key in obj ? __defProp$1(obj, key, { enumerable: true, configurable: true, writable: true, value }) : obj[key] = value;
var __spreadValues = (a, b) => {
  for (var prop in b || (b = {}))
    if (__hasOwnProp.call(b, prop))
      __defNormalProp(a, prop, b[prop]);
  if (__getOwnPropSymbols)
    for (var prop of __getOwnPropSymbols(b)) {
      if (__propIsEnum.call(b, prop))
        __defNormalProp(a, prop, b[prop]);
    }
  return a;
};
const _UUIEvent = class extends Event {
  constructor(evName, eventInit = {}) {
    super(evName, __spreadValues(__spreadValues({}, _UUIEvent.defaultInit), eventInit));
    this.detail = eventInit.detail || {};
  }
};
let UUIEvent = _UUIEvent;
UUIEvent.defaultInit = {
  composed: true
};

class UUIBooleanInputEvent extends UUIEvent {
}
UUIBooleanInputEvent.CHANGE = "change";

var __defProp = Object.defineProperty;
var __getOwnPropDesc = Object.getOwnPropertyDescriptor;
var __decorateClass = (decorators, target, key, kind) => {
  var result = kind > 1 ? void 0 : kind ? __getOwnPropDesc(target, key) : target;
  for (var i = decorators.length - 1, decorator; i >= 0; i--)
    if (decorator = decorators[i])
      result = (kind ? decorator(target, key, result) : decorator(result)) || result;
  if (kind && result)
    __defProp(target, key, result);
  return result;
};
class UUIBooleanInputBaseElement extends LabelMixin("", LitElement) {
  constructor(inputRole = "checkbox") {
    super();
    this._value = "on";
    this.name = "";
    this.labelPosition = "right";
    this.hideLabel = false;
    this.error = false;
    this._checked = false;
    this.disabled = false;
    this.inputRole = inputRole;
    this._internals = this.attachInternals();
  }
  get value() {
    return this._value;
  }
  set value(newVal) {
    const oldValue = this._value;
    this._value = newVal;
    this._internals.setFormValue(this._checked && this.name !== "" ? this._value : null);
    this.requestUpdate("value", oldValue);
  }
  get checked() {
    return this._checked;
  }
  set checked(newVal) {
    const oldValue = this._checked;
    this._checked = newVal;
    this._internals.setFormValue(this._checked && this.name !== "" ? this._value ? this._value : "on" : null);
    this.requestUpdate("checked", oldValue);
  }
  _onInputChange() {
    this.checked = this._input.checked;
    this.dispatchEvent(new UUIBooleanInputEvent(UUIBooleanInputEvent.CHANGE, { bubbles: true }));
  }
  render() {
    return html`
      <label>
        <input
          type="checkbox"
          id="input"
          ?disabled="${this.disabled}"
          @change="${this._onInputChange}"
          .checked="${this.checked}"
          aria-checked="${this.checked ? "true" : "false"}"
          aria-label=${this.label}
          role="${this.inputRole}" />
        ${this.renderCheckbox()}
        ${this.hideLabel === false ? this.renderLabel() : ""}
      </label>
    `;
  }
}
UUIBooleanInputBaseElement.styles = [
  css`
      :host {
        display: inline-block;
      }

      label {
        cursor: pointer;
        -webkit-user-select: none;
           -moz-user-select: none;
            -ms-user-select: none;
                user-select: none;

        display: flex;
        flex-wrap: nowrap;
        align-items: center;
        justify-items: center;
        gap: var(--uui-size-xsmall,9px);
      }

      input {
        position: absolute;
        height: 0px;
        width: 0px;
      }

      :host([label-position='left']) label {
        flex-direction: row-reverse;
      }

      :host([label-position='top']) label {
        gap: var(--uui-size-half-base-unit,3px);
        flex-direction: column-reverse;
      }

      :host([label-position='bottom']) label {
        gap: var(--uui-size-half-base-unit,3px);
        flex-direction: column;
      }

      :host([disabled]) .label {
        opacity: 0.5;
      }
    `
];
UUIBooleanInputBaseElement.formAssociated = true;
__decorateClass([
  query("#input")
], UUIBooleanInputBaseElement.prototype, "_input", 2);
__decorateClass([
  property({ type: String })
], UUIBooleanInputBaseElement.prototype, "value", 1);
__decorateClass([
  property({ type: String })
], UUIBooleanInputBaseElement.prototype, "name", 2);
__decorateClass([
  property({ type: String, attribute: "label-position", reflect: true })
], UUIBooleanInputBaseElement.prototype, "labelPosition", 2);
__decorateClass([
  property({ type: Boolean, attribute: "hide-label", reflect: true })
], UUIBooleanInputBaseElement.prototype, "hideLabel", 2);
__decorateClass([
  property({ type: Boolean, reflect: true })
], UUIBooleanInputBaseElement.prototype, "error", 2);
__decorateClass([
  property({ type: Boolean, reflect: true })
], UUIBooleanInputBaseElement.prototype, "checked", 1);
__decorateClass([
  property({ type: Boolean, reflect: true })
], UUIBooleanInputBaseElement.prototype, "disabled", 2);

export { UUIBooleanInputBaseElement };
